/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.db;

import org.opengion.fukurou.system.LogWriter;
import org.opengion.hayabusa.resource.ResourceFactory;				// 6.2.6.0 (2015/06/19)
import org.opengion.hayabusa.resource.ResourceManager;				// 6.2.6.0 (2015/06/19)
import org.opengion.fukurou.util.StringUtil;
import static org.opengion.fukurou.system.HybsConst.CR ;				// 6.1.0.0 (2014/12/26)
import static org.opengion.fukurou.system.HybsConst.BUFFER_MIDDLE;	// 6.1.0.0 (2014/12/26) refactoring

/**
 * データのコード情報を取り扱うクラスです。
 *
 * 文字列の 「キー:ラベル キー:ラベル」の情報から、HTMLのメニューやリストを作成するための 
 * オプションタグを作成したり、与えられたキーをもとに、チェック済みのオプションタグを
 * 作成したりします。
 * ラベル にスペースを含ませる場合は、ダブルクォーテーションで囲ってください。
 *
 * @og.rev 5.6.6.0 (2013/07/05) 新規追加
 * @og.rev 5.7.7.1 (2014/06/13) Selection_NULL を 基本実装とします。
 * @og.group 選択データ制御
 *
 * @version  4.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class Selection_KEYVAL extends Selection_NULL {
	private final String	ORG_KEYVAL ;

	private final String	CACHE ;

	/**
	 * コンストラクター
	 *
	 * 既存の、CodeData には存在しない、新しいコードリソースを作成する為の、文字列を指定します。
	 * 文字列は、「キー:ラベル キー:ラベル」形式で、スペースで分解後、":" でキーとラベルに分離します。
	 * スペース分解後の文字列に、":" が含まれていない場合は、キーをラベルとして扱います。
	 * また、ラベル部分は、ラベルリソースを使用して、変換を行います。
	 * 内部的には、CodeData を作成しません。DBColumnオブジェクト内で、直接、Selection_KEYVAL を生成します。
	 * codeName 、codeList、codeGroup などが指定された場合は、そちらが優先されます。
	 * 「キー:ラベル キー:ラベル」で、ラベル にスペースを含ませる場合は、ダブルクォーテーションで囲ってください。
	 * 「"キー:ラベル" "キー:ラベル"」という感じです。
	 *
	 * @og.rev 5.6.7.1 (2013/08/09) 「キー:ラベル キー:ラベル」分解に、クオート処理を加味
	 * @og.rev 6.2.6.0 (2015/06/19) type別Selectionの場合、ラベルリソースを使用する為、言語を引数で渡す。
	 *
	 * @param	strCode	コードデータパラメータ文字列
	 * @param	lang  言語
	 */
	public Selection_KEYVAL( final String strCode,final String lang ) {
		super();		// 6.4.1.1 (2016/01/16) PMD refactoring. It is a good practice to call super() in a constructor
		ORG_KEYVAL = strCode ;

		// 6.2.6.0 (2015/06/19)
		final ResourceManager resource = ResourceFactory.newInstance( lang );

		// 6.2.6.0 (2015/06/19) 仕様変更。コロン が無ければ、キーからラベルを求める。
		// 6.4.1.1 (2016/01/16) PMD refactoring. Avoid if (x != y) ..; else ..;
		if( strCode == null || strCode.isEmpty() ) {
			CACHE = "";
		}
		else {
			final String[] keyvals = StringUtil.csv2Array( strCode, ' ' );	// 5.6.7.1 (2013/08/09) クオート処理を加味
			final int size = keyvals.length;

			final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );
			for( int i=0; i<size; i++ ) {
				final String keyval = keyvals[i] ;
				if( keyval == null || keyval.length() <= 0 ) { continue; }
				final int idx = keyval.indexOf( ':' );
				// 6.2.6.0 (2015/06/19) 仕様変更。コロン が無ければ、キーからラベルを求める。

				final String key = idx < 0 ? keyval : keyval.substring( 0,idx ).trim();
				final String val = resource.getLabel( 
										idx < 0 ? keyval : keyval.substring( idx+1 ).trim()
									);

				// 6.0.2.5 (2014/10/31) char を append する。
				buf.append( "<option value=\"" ).append( key )
					.append( "\">" ).append( val ).append( "</option>" );
			}

			CACHE = buf.toString();
		}
	}

	/**
	 * 初期値が選択済みの 選択肢(オプション)を返します。
	 * このオプションは、引数の値を初期値とするオプションタグを返します。
	 * このクラスでは、useShortLabel は、無視されます。(常に、false です)
	 *
	 * @param   selectValue  選択されている値
	 * @param   seqFlag  シーケンスアクセス機能 [true:ON/false:OFF]
	 * @param   useShortLabel ラベル(短)をベースとしたオプション表示を行うかどうか(常にfalse)。
	 *
	 * @return  オプションタグ
	 * @og.rtnNotNull
	 */
	@Override
	public String getOption( final String selectValue,final boolean seqFlag, final boolean useShortLabel ) {
		// マッチするアドレスを探す。キーの前後のダブルクオートを加味して検索
		final String selVal = "\"" + selectValue + "\"" ;

		final int indx = CACHE.indexOf( selVal );

		if( indx < 0 ) {
			// 4.0.0 (2005/01/31)
			if( selectValue != null && selectValue.length() > 0 ) {
				final String errMsg = "コードに存在しない値が指定されました。"
							+ " value=[" + selectValue + "]"
							+ CR + ORG_KEYVAL ;
				LogWriter.log( errMsg );
			}
			return CACHE;
		}
		else {
			final int addIndx = indx + selVal.length() ;	// selected の挿入位置

			final StringBuilder buf = new StringBuilder( BUFFER_MIDDLE );
			// 3.6.0.6 (2004/10/22) シーケンスアクセス機能を指定する seqFlag を導入
			if( seqFlag ) {
				buf.append( "<option value=\"" ).append( selectValue ).append( '"' );		// 6.0.2.5 (2014/10/31) char を append する。
			}
			else {
				buf.append( CACHE.substring( 0,addIndx ) );
			}
			buf.append( " selected=\"selected\"" )
				.append( CACHE.substring( addIndx ) );
			return buf.toString() ;
		}
	}

	/**
	 * 選択肢(value)に対するラベルを返します。
	 * 選択肢(value)が、存在しなかった場合は、選択肢そのものを返します。
	 * getValueLabel( XX,false ) は、getValueLabel( XX ) と同じです。
	 *
	 * ※ このクラスでは、短縮ラベルは使用されません。
	 *
	 * @param	selectValue	選択肢の値
	 * @param	isSLbl	短縮ラベルを [true:使用する/false:しない](常に false)
	 *
	 * @return  選択肢のラベル
	 * @see     #getValueLabel( String )
	 */
	@Override
	public String getValueLabel( final String selectValue,final boolean isSLbl ) {
		// マッチするアドレスを探す。キーの前後のダブルクオートを加味して検索
		final String selVal = "\"" + selectValue + "\"" ;

		final int indx = CACHE.indexOf( selVal );

		if( indx < 0 ) {
			// マッチしなければ、選択肢そのものを返す。
			return selectValue;
		}
		else {
			// マッチすれば、キー以下のBODY部の文字列を切り出して返す。
			final int stIdx = indx + selVal.length() + 1 ;			// ＋１ は、">" の位置
			final int edIdx = CACHE.indexOf( '<',stIdx );				// 終了アドレス

			return CACHE.substring( stIdx,edIdx );
		}
	}

}
